#include <string.h>
#include <malloc.h>

#include "Messenger.h"

#include <Application.h>
#include <Roster.h>
#include <Message.h>
#include <InterfaceDefs.h>
#include <Window.h>

#include "EasyMoveFilter.h"
#include "interface.h"

const bigtime_t HUGE_TIMEOUT = 1000000;

EasyMoveFilter::EasyMoveFilter() :
	mWinMsgr(0),
	mValidFrame(false),
	mValidLook(false),
	mIsMoving(false),
	init(B_OK)
{
	updateThread=spawn_thread(UpdateThreadFunc, "updateThread", B_NORMAL_PRIORITY, this);
	if(updateThread>=0)
		resume_thread(updateThread);
	
	oldWorkspace=current_workspace();
}

EasyMoveFilter::~EasyMoveFilter()
{
	if(mWinMsgr)
		delete mWinMsgr;
}

status_t EasyMoveFilter::InitCheck()
{
	return init;
}

filter_result EasyMoveFilter::Filter(BMessage *message, BList *outList)
{
	BPoint where;
	filter_result ret=B_DISPATCH_MESSAGE;
	
	int32 ws;
	if(mWinMsgr && (mOldWorkspace!=(ws=current_workspace())))
	{
		BMessage msg(B_SET_PROPERTY);
		msg.AddSpecifier("Frame");
		msg.AddRect("data", mWinFrame);
		mWinMsgr->SendMessage(&msg, (BHandler *)0, HUGE_TIMEOUT);
		
		mOldWorkspace=ws;
	}
	
	switch(message->what)
	{
		case B_MOUSE_DOWN:
			if(message->FindInt32("buttons"  , &buttons  )!=B_OK) break;
			if(message->FindInt32("modifiers", &modifiers)!=B_OK) break;
			if(message->FindPoint("where"    , &where    )!=B_OK) break;
			
			mMousePos=where;
			
			// bring to front
			if(((((uint32)modifiers) & (B_SHIFT_KEY | B_COMMAND_KEY | B_CONTROL_KEY | B_OPTION_KEY | B_MENU_KEY))==prefs.GetFrontModifierMask()) && (buttons & prefs.GetFrontButtonMask()))
			{
				if(!mWinMsgr)
					mWinMsgr=FindWindow(where);
				if(mWinMsgr)
				{
					if(mIsFrontMost && prefs.GetToggleFront())
						PutWinToBack();
					else
						BringToFront();
					
					delete mWinMsgr;
					mWinMsgr=0;
					ret=B_SKIP_MESSAGE;
				}
			}
			
			// put to back
			if(((((uint32)modifiers) & (B_SHIFT_KEY | B_COMMAND_KEY | B_CONTROL_KEY | B_OPTION_KEY | B_MENU_KEY))==prefs.GetBackModifierMask()) && (buttons & prefs.GetBackButtonMask()))
			{
				if(!mWinMsgr)
					mWinMsgr=FindWindow(where);
				if(mWinMsgr)
				{
					if(mIsBackMost && prefs.GetToggleBack())
						BringToFront();
					else
						PutWinToBack();
					delete mWinMsgr;
					mWinMsgr=0;
					ret=B_SKIP_MESSAGE;
				}
			}
			
			// move window
			if(((((uint32)modifiers) & (B_SHIFT_KEY | B_COMMAND_KEY | B_CONTROL_KEY | B_OPTION_KEY | B_MENU_KEY))==prefs.GetMoveModifierMask()) && (buttons & prefs.GetMoveButtonMask()))
			{
				if(!mWinMsgr)
					mWinMsgr=FindWindow(where);
				if(mWinMsgr)
				{
					mIsMoving=true;
					mOldWorkspace=current_workspace();
					SetWinWorkspaces(-1);
					ret=B_SKIP_MESSAGE;
					mRateCounter=0;
					mRateCeil=0;
				}
			}
			
			// resize window
			if(((((uint32)modifiers) & (B_SHIFT_KEY | B_COMMAND_KEY | B_CONTROL_KEY | B_OPTION_KEY | B_MENU_KEY))==prefs.GetResizeModifierMask()) && (buttons & prefs.GetResizeButtonMask()))
			{
				if(!mWinMsgr)
					mWinMsgr=FindWindow(where);
				if(mWinMsgr)
				{
					mResizingDirection=0;
					ret=B_SKIP_MESSAGE;
					mRateCounter=0;
					mRateCeil=0;
				}
			}
			break;
			
		case B_MOUSE_MOVED:
			if(message->FindPoint("where"  , &where  )!=B_OK) break;
			if(mWinMsgr && mValidFrame)
				if(mIsMoving)
				{
					if(!prefs.GetAnyWin())
						if(mWinFlags & B_NOT_MOVABLE)
							break;
					mWinFrame.OffsetBy(where-oldPos);
					MoveWin();
				}
				else
				{
					bool resizeH=true;
					bool resizeV=true;
					
					if(!prefs.GetAnyWin())
					{
						resizeH=!(mWinFlags & (B_NOT_H_RESIZABLE | B_NOT_RESIZABLE));
						resizeV=!(mWinFlags & (B_NOT_V_RESIZABLE | B_NOT_RESIZABLE));
						
						if(mValidLook)
							if((mWinLook!=B_TITLED_WINDOW_LOOK) && (mWinLook!=B_DOCUMENT_WINDOW_LOOK) && (mWinLook!=B_MODAL_WINDOW_LOOK) && (mWinLook!=B_FLOATING_WINDOW_LOOK))
								resizeH=resizeV=false;
					}
					
					if(resizeH)
					{
						if((where.x<=mWinFrame.left) && !(mResizingDirection & RIGHT))
							mResizingDirection|=LEFT;
						if((where.x>=mWinFrame.right) && !(mResizingDirection & LEFT))
							mResizingDirection|=RIGHT;
					}
					if(resizeV)
					{
						if((where.y<=mWinFrame.top) && !(mResizingDirection & BOTTOM))
							mResizingDirection|=TOP;
						if((where.y>=mWinFrame.bottom) && !(mResizingDirection & TOP))
							mResizingDirection|=BOTTOM;
					}
					if(mResizingDirection & LEFT)
						mWinFrame.left=where.x;
					if(mResizingDirection & RIGHT)
						mWinFrame.right=where.x;
					if(mResizingDirection & TOP)
						mWinFrame.top=where.y;
					if(mResizingDirection & BOTTOM)
						mWinFrame.bottom=where.y;
					MoveWin();
				}
			oldPos=where;
			break;
			
		case B_MOUSE_UP:
			if(mWinMsgr)
			{
				if(mIsMoving)
				{
					int32 workspace=current_workspace();
					if(!(mWinWorkspaceMask & 1<<workspace))
					{
						mWinWorkspaceMask&=!(1<<mOldWorkspace);
						mWinWorkspaceMask|=  1<<workspace;
					}
					SetWinWorkspaces(mWinWorkspaceMask);
					mIsMoving=false;
				}
				
				mValidFrame=false;
				mValidLook=false;
				delete mWinMsgr;
				mWinMsgr=0;
				ret=B_SKIP_MESSAGE;
			}
			break;
	};
	
	return ret;
}

void EasyMoveFilter::PutWinToBack()
{
	BMessage reply;
	BMessage msgGet(B_GET_PROPERTY);
	msgGet.AddSpecifier("Feel");
	mWinMsgr->SendMessage(&msgGet, &reply, HUGE_TIMEOUT, HUGE_TIMEOUT);
	int32 originalFeel;
	if(reply.FindInt32("result", &originalFeel)!=B_OK)
		return;
	
	BMessage msgSet(B_SET_PROPERTY);
	msgSet.AddSpecifier("Feel");
	msgSet.AddInt32("data", 1024);
	mWinMsgr->SendMessage(&msgSet, (BHandler *)0, HUGE_TIMEOUT);
	
	msgSet.ReplaceInt32("data", originalFeel);
	mWinMsgr->SendMessage(&msgSet, (BHandler *)0, HUGE_TIMEOUT);
}

void EasyMoveFilter::BringToFront()
{
	mode_mouse oldMode=mouse_mode();
	if((oldMode==B_WARP_MOUSE) || (oldMode==B_INSTANT_WARP_MOUSE))
	{
		set_mouse_mode(B_FOCUS_FOLLOWS_MOUSE);
		do_window_action(mWinToken, B_BRING_TO_FRONT, BRect(0, 0, 0, 0), false);
		set_mouse_mode(oldMode);
	}
	else
		do_window_action(mWinToken, B_BRING_TO_FRONT, BRect(0, 0, 0, 0), false);
}

void EasyMoveFilter::SetWinWorkspaces(int32 ws)
{
	if(!mWinMsgr)
		return;
	
	BMessage workspace_msg(B_SET_PROPERTY);
	workspace_msg.AddSpecifier("Workspaces");
	workspace_msg.AddInt32("data", ws);
	mWinMsgr->SendMessage(&workspace_msg, (BHandler *)0, HUGE_TIMEOUT);
}

void EasyMoveFilter::MoveWin()
{
	mRateCounter+=prefs.GetUpdateRate();
	if(mRateCounter>mRateCeil)
	{
		BMessage msg(B_SET_PROPERTY);
		msg.AddSpecifier("Frame");
		msg.AddRect("data", mWinFrame);
		mWinMsgr->SendMessage(&msg, (BHandler *)0, HUGE_TIMEOUT);
		
		mRateCeil=ceil(mRateCounter);
	}
}

BMessenger *EasyMoveFilter::FindWindow(BPoint pos)
{
	int32 *tokens;
	int32 count;
	
	tokens=get_token_list(-1, &count);
	if(!tokens)
		return 0;
	
	for(int32 i=0; i<count; i++)
	{
		window_info *wi;
		wi=get_window_info(tokens[i]);
		if(!wi)
			continue;
		if(wi->layer<=2)
		{
			free(wi);
			free(tokens);
			return 0;
		}
		if((wi->window_left<=pos.x) && (wi->window_right>=pos.x) &&
		   (wi->window_top<=pos.y) && (wi->window_bottom>=pos.y))
		{
			BMessenger *msgr=new BMessenger;
			msgr->fPort=wi->client_port;
			msgr->fHandlerToken=wi->client_token;
			msgr->fTeam=wi->team;
			mWinToken=tokens[i];
			mWinWorkspaceMask=wi->workspaces;
			mWinFlags=wi->flags;
			mIsFrontMost=!i;
			
			if(i<count-1)
			{
				window_info *wi2=get_window_info(tokens[i+1]);
				mIsBackMost=wi2->layer<=2;
				free(wi2);
			}
			else
				mIsBackMost=false;
			
			free(wi);
			free(tokens);
			
			BMessage msg(B_GET_PROPERTY);
			BMessage reply;
			msg.AddSpecifier("Frame");
			msgr->SendMessage(&msg, &reply, HUGE_TIMEOUT, HUGE_TIMEOUT);
			mValidFrame=(reply.FindRect("result", &mWinFrame)==B_OK);
			
			BMessage msg2(B_GET_PROPERTY);
			BMessage reply2;
			msg2.AddSpecifier("Look");
			msgr->SendMessage(&msg2, &reply2, HUGE_TIMEOUT, HUGE_TIMEOUT);
			mValidLook=(reply2.FindInt32("result", (int32 *)&mWinLook)==B_OK);
			return msgr;
		}
		free(wi);
	}
	
	free(tokens);
	return 0;
}

status_t EasyMoveFilter::UpdateThreadFunc(void *cookie)
{
	port_id updatePrefsPort=create_port(1, "EasyMove UpdatePrefsPort");
	if(updatePrefsPort<0)
		return updatePrefsPort;
	
	while(1)
	{
		int32 msg_code;
		int8 buff[10];
		status_t err=read_port(updatePrefsPort, &msg_code, buff, 10);
		if(err<0)
			return err;
		((EasyMoveFilter *)cookie)->prefs.Update();
	}
	
	return B_OK;
}
