#include "ModifierBox.h"
#include "ModifierView.h"

const ModifierBox::MaskListType ModifierBox::ModifierList[]=
{
	{B_SHIFT_KEY,	"Shift"},
	{B_COMMAND_KEY,	"Command"},
	{B_CONTROL_KEY,	"Control"},
	{B_OPTION_KEY,	"Option"},
	{B_MENU_KEY,	"Menu"}
};

const int32 ModifierBox::NbModifiers=5;

const ModifierBox::MaskListType ModifierBox::ButtonList[]=
{
	{B_PRIMARY_MOUSE_BUTTON,	"Primary mouse button"},
	{B_SECONDARY_MOUSE_BUTTON,	"Secondary mouse button"},
	{B_TERTIARY_MOUSE_BUTTON,	"Tertiary mouse button"}
};

const int32 ModifierBox::NbButtons=3;

ModifierBox::ModifierBox(BRect frame, const char *label, uint32 modifier, uint32 button) :
	BBox(frame),
	mDefaultModifier(0),
	mDefaultButton(0)
{
	if(label)
		SetLabel(label);
	
	float right=0;
	float bottom;
	BRect frame(10, 15, 10, 15);
	
	mModifierViewList=new ModifierView*[NbModifiers];
	
	for(int32 i=0; i<NbModifiers; i++)
	{
		mModifierViewList[i]=new ModifierView(frame,
											  ModifierList[i].mask,
											  ModifierList[i].label,
											  ModifierList[i].mask & modifier,
											  new BMessage(MODIFIER_CHANGED));
		mModifierViewList[i]->ResizeToPreferred();
		AddChild(mModifierViewList[i]);
		right=max_c(right, mModifierViewList[i]->Frame().right);
		frame.OffsetTo(frame.left, mModifierViewList[i]->Frame().bottom+5);
	}
	
	mButtonViewList=new ModifierView*[NbButtons];
	frame=BRect(right+10, 15, right+10, 15);
	
	for(int32 i=0; i<NbButtons; i++)
	{
		mButtonViewList[i]=new ModifierView(frame,
										    ButtonList[i].mask,
										    ButtonList[i].label,
										    ButtonList[i].mask & button,
										    new BMessage(BUTTON_CHANGED));
		mButtonViewList[i]->ResizeToPreferred();
		AddChild(mButtonViewList[i]);
		right=max_c(right, mButtonViewList[i]->Frame().right);
		frame.OffsetTo(frame.left, mButtonViewList[i]->Frame().bottom+5);
	}
	bottom=mModifierViewList[NbModifiers-1]->Frame().bottom;
	ResizeTo(right+10, bottom+10);
}

ModifierBox::~ModifierBox()
{
	delete mModifierViewList;
	delete mButtonViewList;
}

void ModifierBox::MessageReceived(BMessage *message)
{
	uint32 mask=0;
	
	switch(message->what)
	{
		case MODIFIER_CHANGED:
		{
			for(int32 i=0; i<NbModifiers; i++)
				mask|=mModifierViewList[i]->GetMask();
			if(!mask)
			{
				mask=mDefaultModifier;
				for(int32 i=0; i<NbModifiers; i++)
					mModifierViewList[i]->SetValue(ModifierList[i].mask & mask);
			}
			InvokeNotify(Message());
			break;
		}
		
		case BUTTON_CHANGED:
			for(int32 i=0; i<NbButtons; i++)
				mask|=mButtonViewList[i]->GetMask();
			if(!mask)
			{
				mask=mDefaultButton;
				for(int32 i=0; i<NbButtons; i++)
					mButtonViewList[i]->SetValue(ButtonList[i].mask & mask);
			}
			InvokeNotify(Message());
			break;
		default:
			BBox::MessageReceived(message);
	}
}

void ModifierBox::AttachedToWindow()
{
	for(int32 i=0; i<NbModifiers; i++)
		mModifierViewList[i]->SetTarget(this);
	for(int32 i=0; i<NbButtons; i++)
		mButtonViewList[i]->SetTarget(this);
}

uint32 ModifierBox::GetModifierMask()
{
	uint32 mask=0;
	for(int32 i=0; i<NbModifiers; i++)
		mask|=mModifierViewList[i]->GetMask();
	return mask;
}

uint32 ModifierBox::GetButtonMask()
{
	uint32 mask=0;
	for(int32 i=0; i<NbButtons; i++)
		mask|=mButtonViewList[i]->GetMask();
	return mask;
}

void ModifierBox::SetModifierMask(uint32 mask)
{
	for(int32 i=0; i<NbModifiers; i++)
		mModifierViewList[i]->SetValue(ModifierList[i].mask & mask);
}

void ModifierBox::SetButtonMask(uint32 mask)
{
	for(int32 i=0; i<NbButtons; i++)
		mButtonViewList[i]->SetValue(ButtonList[i].mask & mask);
}

void ModifierBox::SetDefaultModifierMask(uint32 mask)
{
	mDefaultModifier=mask;
}

void ModifierBox::SetDefaultButtonMask(uint32 mask)
{
	mDefaultButton=mask;
}
