#include <iostream>
#include <string.h>

#include <storage/Path.h>
#include <storage/File.h>
#include <storage/Directory.h>
#include <storage/FindDirectory.h>
#include <interface/View.h>
#include <interface/InterfaceDefs.h>

#include "Preferences.h"

Preferences::Preferences()
{
	Load();
}

Preferences::~Preferences()
{
	Save();
}

void Preferences::Update()
{
	Load();
}

void Preferences::SetToDefault()
{
	SetMoveModifierMask (GetDefaultMoveModifierMask());
	SetMoveButtonMask   (GetDefaultMoveModifierMask());
	SetFrontModifierMask(GetDefaultMoveModifierMask());
	SetFrontButtonMask  (GetDefaultMoveModifierMask());
	SetBackModifierMask (GetDefaultMoveModifierMask());
	SetBackButtonMask   (GetDefaultMoveModifierMask());
	SetToggleFront      (GetDefaultMoveModifierMask());
	SetToggleBack       (GetDefaultMoveModifierMask());
	SetUpdateRate       (GetDefaultMoveModifierMask());
	
	SetToggleFrontToDefault();
	SetToggleBackToDefault();
	SetAnyWinToDefault();
	SetUpdateRateToDefault();
}

BFile *Preferences::OpenFile(uint32 openMode)
{
	BPath path;
	find_directory(B_USER_SETTINGS_DIRECTORY, &path);
	BDirectory dir(path.Path());
	BFile *file=new BFile(&dir, PrefsFilename, openMode);
	status_t err;
	err=file->InitCheck();
	if(err!=B_OK)
	{
		if(openMode&B_CREATE_FILE)
			cerr <<"could not open file: " <<strerror(err) <<endl;
		delete file;
		return 0;
	}
	return file;
}

void Preferences::Load()
{
	SetToDefault();
	BFile *file=OpenFile(B_READ_ONLY);
	if(!file)
		return;
	file->Read(&mMoveModifierMask,   sizeof(uint32));
	file->Read(&mMoveButtonMask,     sizeof(uint32));
	file->Read(&mFrontModifierMask,  sizeof(uint32));
	file->Read(&mFrontButtonMask,    sizeof(uint32));
	file->Read(&mBackModifierMask,   sizeof(uint32));
	file->Read(&mBackButtonMask,     sizeof(uint32));
	file->Read(&mToggleFront,        sizeof(bool));
	file->Read(&mToggleBack,         sizeof(bool));
	file->Read(&mRate,               sizeof(float));
	file->Read(&mResizeModifierMask, sizeof(uint32));
	file->Read(&mResizeButtonMask,   sizeof(uint32));
	file->Read(&mAnyWin,             sizeof(bool));
	delete file;
}

void Preferences::Save()
{
	BFile *file=OpenFile(B_WRITE_ONLY | B_CREATE_FILE);
	if(!file)
		return;
	
	file->Write(&mMoveModifierMask,   sizeof(uint32));
	file->Write(&mMoveButtonMask,     sizeof(uint32));
	file->Write(&mFrontModifierMask,  sizeof(uint32));
	file->Write(&mFrontButtonMask,    sizeof(uint32));
	file->Write(&mBackModifierMask,   sizeof(uint32));
	file->Write(&mBackButtonMask,     sizeof(uint32));
	file->Write(&mToggleFront,        sizeof(bool));
	file->Write(&mToggleBack,         sizeof(bool));
	file->Write(&mRate,               sizeof(float));
	file->Write(&mResizeModifierMask, sizeof(uint32));
	file->Write(&mResizeButtonMask,   sizeof(uint32));
	file->Write(&mAnyWin,             sizeof(bool));
	
	delete file;
}
