#include <stdlib.h>

#include <Application.h>
#include <TextControl.h>
#include <TextView.h>
#include <String.h>
#include <View.h>
#include <Box.h>

#include "ModifierView.h"
#include "SettingsWin.h"
#include "ModifierBox.h"

const SettingsWin::MaskListType SettingsWin::ModifierList[]=
{
	{B_SHIFT_KEY,	"Shift"},
	{B_COMMAND_KEY,	"Command"},
	{B_CONTROL_KEY,	"Control"},
	{B_OPTION_KEY,	"Option"},
	{B_MENU_KEY,	"Menu"}
};

const int32 SettingsWin::NbModifiers=5;

const SettingsWin::MaskListType SettingsWin::ButtonList[]=
{
	{B_PRIMARY_MOUSE_BUTTON,	"Primary mouse button"},
	{B_SECONDARY_MOUSE_BUTTON,	"Secondary mouse button"},
	{B_TERTIARY_MOUSE_BUTTON,	"Tertiary mouse button"}
};

const int32 SettingsWin::NbButtons=3;

SettingsWin::SettingsWin() :
	BWindow(BRect(100, 100, 400, 300), "EasyMove Settings", B_TITLED_WINDOW, B_ASYNCHRONOUS_CONTROLS | B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	bg=new BView(BRect(Bounds()), "", B_FOLLOW_ALL_SIDES, B_WILL_DRAW);
	bg->SetViewColor(216, 216, 216);
	AddChild(bg);
	
	BTextView *textView=new BTextView(BRect(10, 10, Bounds().right-10, 55), "", BRect(10, 10, Bounds().right-10, 50), B_FOLLOW_NONE, B_WILL_DRAW);
	textView->MakeEditable(false);
	textView->MakeSelectable(false);
	textView->SetWordWrap(false);
	textView->SetViewColor(216, 216, 216);
	textView->SetText("EasyMove will work if all the modifiers are\npresent, and one of the buttons pushed.");
	textView->ResizeToPreferred();
	bg->AddChild(textView);
	
	float right;
	float bottom;
	BRect frame;
	
	frame=BRect(10, textView->Frame().bottom+10, 10, textView->Frame().bottom+10);
	
	mMoveBox=new ModifierBox(frame,
							 "Move window",
							 mPrefs.GetMoveModifierMask(),
							 mPrefs.GetMoveButtonMask());
	mMoveBox->SetDefaultModifierMask(mPrefs.GetDefaultMoveModifierMask());
	mMoveBox->SetDefaultButtonMask  (mPrefs.GetDefaultMoveButtonMask());
	mMoveBox->SetMessage(new BMessage(MOVE_CHANGED));
	mMoveBox->SetTarget(this);
	bg->AddChild(mMoveBox);
	
	mResizeBox=new ModifierBox(frame.OffsetByCopy(0, mMoveBox->Bounds().Height()+10),
							   "Resize window",
							   mPrefs.GetResizeModifierMask(),
							   mPrefs.GetResizeButtonMask());
	mResizeBox->SetDefaultModifierMask(mPrefs.GetDefaultResizeModifierMask());
	mResizeBox->SetDefaultButtonMask  (mPrefs.GetDefaultResizeButtonMask());
	mResizeBox->SetMessage(new BMessage(RESIZE_CHANGED));
	mResizeBox->SetTarget(this);
	bg->AddChild(mResizeBox);
	
	mFrontBox=new ModifierBox(frame.OffsetByCopy(mMoveBox->Bounds().Width()+10, 0),
							  "Bring to front",
							  mPrefs.GetFrontModifierMask(),
							  mPrefs.GetFrontButtonMask());
	mFrontBox->SetDefaultModifierMask(mPrefs.GetDefaultFrontModifierMask());
	mFrontBox->SetDefaultButtonMask  (mPrefs.GetDefaultFrontButtonMask());
	mFrontBox->SetMessage(new BMessage(FRONT_CHANGED));
	mFrontBox->SetTarget(this);
	bg->AddChild(mFrontBox);
	
	mBackBox=new ModifierBox(frame.OffsetByCopy(mMoveBox->Bounds().Width()+10, mMoveBox->Bounds().Height()+10),
							 "Put to back",
							 mPrefs.GetBackModifierMask(),
							 mPrefs.GetBackButtonMask());
	mBackBox->SetDefaultModifierMask(mPrefs.GetDefaultBackModifierMask());
	mBackBox->SetDefaultButtonMask  (mPrefs.GetDefaultBackButtonMask());
	mBackBox->SetMessage(new BMessage(BACK_CHANGED));
	mBackBox->SetTarget(this);
	bg->AddChild(mBackBox);
	
	
	bottom=mBackBox->Frame().bottom;
	right=mBackBox->Frame().right;
	
	mToggleFront=new BCheckBox(BRect(10, mBackBox->Frame().bottom+10, 10, mBackBox->Frame().bottom+10), "mToggleFront", "Put to back if already front most", new BMessage(TOGGLE_FRONT));
	mToggleFront->SetValue(mPrefs.GetToggleFront());
	mToggleFront->ResizeToPreferred();
	bg->AddChild(mToggleFront);
	
	mToggleBack=new BCheckBox(BRect(10, mToggleFront->Frame().bottom+5, 10, mToggleFront->Frame().bottom+5), "mToggleBack", "Bring to front if already back most", new BMessage(TOGGLE_BACK));
	mToggleBack->SetValue(mPrefs.GetToggleBack());
	mToggleBack->ResizeToPreferred();
	bg->AddChild(mToggleBack);
	
	mAnyWindow=new BCheckBox(BRect(10, mToggleBack->Frame().bottom+5, 10, mToggleBack->Frame().bottom+5), "mAnyWindow", "Move and resize any window", new BMessage(ANY_WINDOW));
	mAnyWindow->SetValue(mPrefs.GetAnyWin());
	mAnyWindow->ResizeToPreferred();
	bg->AddChild(mAnyWindow);
	
	mRateText=new BTextControl(BRect(10, mAnyWindow->Frame().bottom+10, 10, mAnyWindow->Frame().bottom+10), "mRateText", "Update rate (in %)", "", new BMessage(PERCENT_CHANGE));
	BString str;
	str <<(float)100.0*mPrefs.GetUpdateRate();
	mRateText->SetText(str.String());
	
	mRateText->ResizeToPreferred();
	mRateText->ResizeTo(150, mRateText->Frame().Height());
	bg->AddChild(mRateText);
	
	bottom=mRateText->Frame().bottom;
	
	ResizeTo(right+10, bottom+10);
}

SettingsWin::~SettingsWin()
{
}

void SettingsWin::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
		case MOVE_CHANGED:
			mPrefs.SetMoveModifierMask(mMoveBox->GetModifierMask());
			mPrefs.SetMoveButtonMask  (mMoveBox->GetButtonMask());
			mPrefs.Save();
			SendUpdateMessage();
			break;
			
		case RESIZE_CHANGED:
			mPrefs.SetResizeModifierMask(mResizeBox->GetModifierMask());
			mPrefs.SetResizeButtonMask  (mResizeBox->GetButtonMask());
			mPrefs.Save();
			SendUpdateMessage();
			break;
			
		case FRONT_CHANGED:
			mPrefs.SetFrontModifierMask(mFrontBox->GetModifierMask());
			mPrefs.SetFrontButtonMask  (mFrontBox->GetButtonMask());
			mPrefs.Save();
			SendUpdateMessage();
			break;
			
		case BACK_CHANGED:
			mPrefs.SetBackModifierMask(mBackBox->GetModifierMask());
			mPrefs.SetBackButtonMask  (mBackBox->GetButtonMask());
			mPrefs.Save();
			SendUpdateMessage();
			break;
			
		case TOGGLE_FRONT:
			mPrefs.SetToggleFront(mToggleFront->Value());
			mPrefs.Save();
			SendUpdateMessage();
			break;
			
		case TOGGLE_BACK:
			mPrefs.SetToggleBack(mToggleBack->Value());
			mPrefs.Save();
			SendUpdateMessage();
			break;
			
		case ANY_WINDOW:
			mPrefs.SetAnyWin(mAnyWindow->Value());
			mPrefs.Save();
			SendUpdateMessage();
			break;
			
		case PERCENT_CHANGE:
		{
			float rate=atof(mRateText->Text());
			if((rate<0) || (rate>100))
				rate=100;
			BString str;
			str <<rate;
			mRateText->SetText(str.String());
			
			mPrefs.SetUpdateRate(rate/100.0);
			mPrefs.Save();
			SendUpdateMessage();
			break;
		}
			
		default:
			BWindow::MessageReceived(message);
	}
}

bool SettingsWin::QuitRequested()
{
	be_app_messenger.SendMessage(B_QUIT_REQUESTED);
	return true;
}

void SettingsWin::SendUpdateMessage()
{
	port_id port=find_port("EasyMove UpdatePrefsPort");
	if(port<0)
		return;
	write_port(port, 0, 0, 0);
}
